/*
 [The "BSD licence"]
 Copyright (c) 2005 Jean Bovet, http://www.cs.usfca.edu/~jbovet/
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
														   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
														   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "PriviledgedSocketCreator.h"

#import "MoreUNIX.h"
#import "MoreSecurity.h"
#import "MoreCFQ.h"

#import "Constants.h"

#include <sys/un.h>
#include <sys/socket.h>
#include <sys/uio.h>

@implementation PriviledgedSocketCreator

- (BOOL)createEmptyAuthorization
{
	int err = AuthorizationCreate (NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, &mAuthorizationRef);
	if(err != noErr) {
		NSLog(@"createEmptyAuthorization: AuthorizationCreate() error %d", err);	
	}
	return err == noErr;
}

- (BOOL)prepareHelperTool
{
	mToolURL = NULL;
	int err = MoreSecCopyHelperToolURLAndCheckBundled(CFBundleGetMainBundle(), 
													  CFSTR("EthernalTool"), 
													  kApplicationSupportFolderType, 
													  CFSTR("Ethernal"), 
													  CFSTR("EthernalTool"),
													  (CFURLRef*)&mToolURL);
	if(err != noErr) {
		NSLog(@"prepareHelperTool: MoreSecCopyHelperToolURLAndCheckBundled() error %d", err);	
	}
	
	return err == noErr;
}

- (BOOL)launchToolAndFetchResultForSlot:(NSString*)slot
{
	NSMutableDictionary *dic = [NSMutableDictionary dictionary];
	[dic setObject:(NSString*)kCommandGetBPFSocket forKey:(NSString*)kCommandNameKey];
	[dic setObject:slot forKey:(NSString*)kSlotNameKey];
	
	CFDictionaryRef response = NULL;
	int err = MoreSecExecuteRequestInHelperTool((CFURLRef)mToolURL, mAuthorizationRef, (CFDictionaryRef)dic, &response);
	if(err != noErr) {
		NSLog(@"launchToolAndFetchResult: MoreSecExecuteRequestInHelperTool() error %d", err);
		return NO;
	}
		
	err = MoreSecGetErrorFromResponse(response);
	if(err != noErr) {
		NSLog(@"launchToolAndFetchResult: MoreSecGetErrorFromResponse() error %d", err);
		return NO;
	}
	
	NSNumber *socketNumber = [[(NSDictionary*)response objectForKey:(NSString*)kMoreSecFileDescriptorsKey] objectAtIndex:0];
	mBPFSocket = [socketNumber intValue];
	return YES;
}

#pragma mark -

- (BOOL)createSocketAtSlot:(NSString*)slot
{
	if(![self createEmptyAuthorization])
		return NO;
	
	if(![self prepareHelperTool])
		return NO;
	
	return [self launchToolAndFetchResultForSlot:slot];	
}

- (int)socket
{
	return mBPFSocket;
}

+ (int)createSocketAtSlot:(NSString*)slot
{
	PriviledgedSocketCreator *creator = [[PriviledgedSocketCreator alloc] init];
	if([creator createSocketAtSlot:slot])
		return [creator socket];
	else
		return -1;
}

@end
